/**
* Script: networking.js
* Written by: Radnen
* Updated: 12/9/2011
**/

/*******************************************/
/* General Purpose Networking Library v1.0 */
/*******************************************/

var Network = {
	port: 1234,
	address: 0,
	data: null,
	isHost: false,
	
	// used if is hosting:
	connections: [],
	infoQueue: [],
	
	/**
	* join(address, port);
	*  - returns true if you can join, and then joins the game.
	**/
	join: function(address, port) {
		this.address = address;
		this.port = port;
		this.data = OpenAddress(address, port);
		return (this.data != null);
	},

	/**
	* host(port);
	*  - returns true if you can host, and will then host the game.
	**/
	host: function(port) {
		this.address = GetLocalAddress();
		this.port = port;
		this.data = ListenOnPort(port);
		this.isHost = true;
		return (this.data != null);
	},
	
	/**
	* connect();
	*  - attempts to connect to the port (call it after join).
	*  - not used if you are the host.
	**/
	connect: function() {
		if (!this.data.isConnected()) {
			if (this.data) this.data.close();
			this.data = OpenAddress(this.address, this.port);
			return false;
		}
		else return true;
	},	
	
	/**
	* listen();
	*  - looks for other clients to join this host.
	*  - used by the host in replace of connect.
	**/
	hostListen: function() {
		if (this.data.isConnected()) {
			this.connections.push(this.data);
			this.data = ListenOnPort(this.port);
		}
	},
	
	/**
	* send(string);
	*  - the data string you wish to send to the host.
	**/
	send: function(string) {
		if (!this.data) return;
		
		var byte_array = CreateByteArrayFromString(string);
		var i = this.connections.length;
		if (this.isHost) {
			while (i--) this.connections[i].write(byte_array);
		}
		else this.data.write(byte_array);
	},
		
	/**
	* receive();
	*  - returns a string of what was sent to it; null if there was nothing.
	*  - it's up to you to process this data.
	**/
	receive: function() {
		if (!this.data) return null;
		
		var byte_array, read_size = this.data.getPendingReadSize();
		if (read_size > 0) {
			byte_array = this.data.read(read_size);
			return CreateStringFromByteArray(byte_array);
		}
		return null;
	},
	
	/**
	* seek();
	*  - fills up infoQueue with data from host's socket.
	*  - host uses this instead of recieveData();
	**/
	seek: function() {
		var byte_array, read_size, i = this.connections.length;
		while (i--) {
			read_size = this.connections[i].getPendingReadSize();
			if (read_size > 0) {
				byte_array = this.connections[i].read(read_size);
				this.infoQueue.push(byte_array);
			}
		}
	},
	
	/**
	* grabData();
	*  - returns a string from it's info queue.
	*  - use this to process incomming data as a host.
	*  - returns null if no data is found.
	**/
	grab: function() {
		if (this.infoQueue.length == 0) return null;
		return CreateStringFromByteArray(this.infoQueue.pop());
	},
}

// test a new model in which each person shares with each other?